unit lexemeListU;

{
  Unit to handle lists of lexemes
}

interface

type

  Tlexemes = ^Tlexeme;

  Tlexeme = record
    lexeme: string;
    next: Tlexemes;
  end;

  { Prototypes for access procedures for lexemeLists }
  function emptyLexemes: Tlexemes; forward;
  procedure destroyLexemes(var lexemes: Tlexemes);                         forward;
  procedure addLexeme(var lexemes: Tlexemes; newLexeme: String);           forward;
  procedure deduceLexemes(var lexemes: Tlexemes; functionDefn: String);    forward;
  procedure reverseLexemes(var lexemes: Tlexemes);                         forward;
  function firstLexeme(lexList: Tlexemes): String;                         forward;
  function restLexemes(lexList: Tlexemes): Tlexemes;                       forward;

{ -----------------------------------------------------------------------------}



implementation

const
   { unitName = 'lexemeListU'; { For debugging }
   unitName = '';


{ Function definitions for lexemeLists}
{ Function to return an empty list of Tlexemes }
function emptyLexemes: Tlexemes;
  begin
    result := nil;
  end;


{ Procedure to destroy a list of lexemes }
procedure destroyLexemes(var lexemes : Tlexemes);
  var
    tempLexemes : Tlexemes;

  begin
    while lexemes <> nil do begin
      tempLexemes := lexemes;
      lexemes := lexemes^.next;
      dispose(tempLexemes);
    end;
  end;


{ Procedure to add a new TbinStream to the front of a TbinStreams }
procedure addLexeme(var lexemes: Tlexemes; newLexeme: String);
  var
    newLexemes: Tlexemes;

  begin
    new(newLexemes);
    newLexemes^.lexeme := newLexeme;
    newLexemes^.next := lexemes;
    lexemes := newLexemes;

    if (unitName <> '') then writeln(unitName + '.addLexeme: Adding <' + newLexeme + '> to list...');

  end;


{ Procedure to form a list of lexemes from a string containing a function definition }
procedure deduceLexemes(var lexemes: Tlexemes; functionDefn: String);
  var
    counter: integer; { Counter }
    flag: integer; { Marker }


  begin
    { Variable initialisation }
    flag := 1;
    { End of variable initialisation }

    for counter := 1 to length(functionDefn) do begin
      if ((functionDefn[counter] = ' ') OR (functionDefn[counter] = '=')) then begin { find the gaps between lexemes }
        addLexeme(lexemes, Copy(functionDefn, flag, counter-flag));
        flag := counter + 1;
        if (functionDefn[counter] = '=') then
          addLexeme(lexemes, '=');
      end;
    end;

    if (flag <= length(functionDefn)) then
      addLexeme(lexemes, Copy(functionDefn, flag, length(functionDefn)));

  end;


{ Procedure to reverse a list of Lexemes }
procedure reverseLexemes(var lexemes : Tlexemes);
  var
    tmpLexemes: Tlexemes;

  begin
    tmpLexemes := emptyLexemes;
    while lexemes <> emptyLexemes do begin
      addLexeme(tmpLexemes, lexemes^.lexeme);
      lexemes := lexemes^.next;
    end;
    lexemes := tmpLexemes;
  end;


{ Function to return the first lexeme in a list of lexemes }
function firstLexeme(lexList : Tlexemes): string;
  begin
    result := lexList^.lexeme;
  end;


{ Function to return a list of lexemes containing all the lexemes except the first one }
function restLexemes(lexList : Tlexemes): Tlexemes;
  begin
    result := lexList^.next;
  end;

{ -----------------------------------------------------------------------------}



end.
 